using System.Timers;
using GraphicsManager.Enums;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using GraphicsManager.Objects.Core.Fonts;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.Common.Input;
using OpenTK.Windowing.GraphicsLibraryFramework;
using Rectangle = GraphicsManager.Objects.Core.Rectangle;
using Timer = System.Timers.Timer;

namespace Luski.GUI.MainScreen.UI.LuskiControls;

public class TextBox : UserControl
{
    private readonly Label BackWatermark;
    private readonly Label BackLabel;
    private readonly Rectangle Pointer;
    private readonly Timer t;

    public int CursorLocation { get; set; }
    
    public TextBox(string DefaultTexture = "Textbox.png")
    :base(Globals.RootWindow.TextureManager.GetTextureResource(DefaultTexture))
    {
	    t = new(500);
	    t.Elapsed += TOnElapsed;
        base.TextureDisplay = TextureDisplay.Center;
        BackLabel = new Label(Globals.DefaultFont)
        {
            HoverMouse = MouseCursor.IBeam,
            IgnoreHover = true
        };
        BackWatermark = new(BackLabel.Font)
        {
	        Text = "Text Box",
            Color = new(128, 128, 128, 255),
            HoverMouse = MouseCursor.IBeam,
            IgnoreHover = true
        };
        Pointer = new()
        {
	        Size = new(1.ScaleInt(), 
		        (int)BackLabel.LineHeight ),
	        Location = BackWatermark.Location,
	        BackgroundColor = Color4.White,
	        Visible = false,
	        IgnoreHover = true
        };
        IgnoreVisForChildren = true;
        Controls.Add(Pointer);
	        
        Controls.Add(BackLabel);
        Controls.Add(BackWatermark);
        WindowLoaded += OnWindowLoaded;
    }

    private Task OnWindowLoaded(RenderObjectCore _)
    {
	    Pointer.Location = BackWatermark.Location;
	    return Task.CompletedTask;
    }

    private void TOnElapsed(object? _, ElapsedEventArgs __)
    {
	    Globals.RootWindow.Invoke(() =>
	    {
		    Pointer.Visible = !Pointer.Visible;
	    });
    }

    public event Func<KeyboardKeyEventArgs, Task>? KeyPress;
    public event Func<Task>? LetterPress;
    public event Func<Task>? LetterRemoved;
    
    public override void UnFocus()
    {
	    base.UnFocus();
	    t.Stop();
	    Pointer.Visible = false;
	    
    }
    public override void Focus()
    {
	    base.Focus();
	    if (Window is null) return;
	    t.Start();
    }

    public override void SendClipEvent(string ClipString)
    {
	    base.SendClipEvent(ClipString);
	    try
	    {
		    Text = Text.Insert(CursorLocation, ClipString);
		    CursorLocation += ClipString.Length;
		    Vector2i f = BackLabel.GetCharLocation(CursorLocation);
		    Pointer.Location = BackLabel.Location + new Vector2i(f.X, f.Y);
		    Pointer.Visible = true;
		    if (LetterPress is not null) LetterPress.Invoke().Wait();
	    }
	    catch
	    {
		    // ignored
	    }
    }
    
    public char? PasswordChar { get => BackLabel.PasswordChar; set => BackLabel.PasswordChar = value; }

    private TextLocation tl = TextLocation.TopLeft;

    public TextLocation TextLocation
    {
	    get => tl;
	    set
	    {
		    tl = value;
		    if (!string.IsNullOrWhiteSpace(Text))
		    {
			    BackLabel.Location = value switch
			    {
				    TextLocation.LineCenter =>  new(10.ScaleInt(),
					    ((Size.Y - BackLabel.Size.Y) / 2)),
				    _ => BackLabel.Location
			    };
			    BackWatermark.Location = BackLabel.Location;
		    }
		    else
		    {
			    BackWatermark.Location = value switch
			    {
				    TextLocation.LineCenter => new(10.ScaleInt(),
					    ((Size.Y - BackWatermark.Size.Y) / 2)),
				    _ => BackWatermark.Location
			    };
			    BackLabel.Location = BackWatermark.Location;
		    }
	    }
    } 
    
    public override bool Visible
    {
        get => base.Visible;
        set 
        {
            if (value == base.Visible) return;
            if (value)
            {
                if (!string.IsNullOrEmpty(BackLabel.Text))
                {
                    BackLabel.Visible = true;
                    BackWatermark.Visible = false;
                }
                else
                {
                    BackLabel.Visible = false;
                    BackWatermark.Visible = true;
                }
            }
            else
            {
                BackLabel.Visible = value;
                BackWatermark.Visible = value;
            }

            base.Visible = value;
        }
    }
    
    public FontInteraction Font { get => BackLabel.Font; }
    public FontInteraction WatermarkFont { get => BackWatermark.Font; }
    
    public Color4 TextColor { get => BackLabel.Color; set => BackLabel.Color = value; }
    public Color4 WatermarkColor { get => BackWatermark.Color; set => BackWatermark.Color = value; }

    public bool AllowMultiLine { get; set; } = true;
    
    public string Text
	{
		get => BackLabel.Text;
		set
		{
			int old = BackLabel.Size.Y;
			BackLabel.Text = value;
			if (!string.IsNullOrEmpty(value))
			{
				bool f = false;
				if (!BackLabel.Visible)
				{
					f = true;
					BackLabel.Visible = true;
					BackLabel.Location = TextLocation switch
					{
						TextLocation.LineCenter => new(10.ScaleInt(), ((Size.Y - BackLabel.Size.Y) / 2)),
						_ => BackLabel.Location
					};
					/*
					_label.Location = TextLocation switch
					{
						TextLocation.TrueCenterLeft => new(Location.X + 5, Location.Y + ((Size.Y - _label.TrueHeight) / 2) - (_label.Size.Y  - _label.TrueHeight), Location.Z),
						TextLocation.PostiveTureCenterLeft => new(Location.X + 5, Location.Y + ((Size.Y - _label.PostiveTrueHeight) / 2) - _label.Size.Y + _label.TrueHeight, Location.Z),
						TextLocation.PxLeft => new(Location.X + 5, Location.Y + ((Size.Y - _label.Size.Y) / 2), Location.Z),
						_ => new(Location.X + 5, Location.Y + 5, Location.Z)
					};*/
					BackWatermark.Location = BackLabel.Location;
				}
				if (BackWatermark.Visible) BackWatermark.Visible = false;
				if (!f && TextLocation == TextLocation.LineCenter && old != BackLabel.Size.Y)
				{
					//_label.Location = new(Location.X + 5, Location.Y + ((Size.Y - _label.TrueHeight) / 2) - (_label.Size.Y - _label.TrueHeight), Location.Z);
					BackWatermark.Location = BackLabel.Location;
				}
			}
			else
			{
				if (BackLabel.Visible) BackLabel.Visible = false;
				if (!BackWatermark.Visible)
				{
					BackWatermark.Visible = true;
					BackWatermark.Location = TextLocation switch
					{
						TextLocation.LineCenter => new(10.ScaleInt(), ((Size.Y - BackWatermark.Size.Y) / 2)),
						_ => BackWatermark.Location
					};
					BackLabel.Location = BackLabel.Location;
				}
			}

			if (BackLabel.Text.Length < CursorLocation)
			{
				CursorLocation = BackLabel.Text.Length;
				var xy = BackLabel.GetCharLocation(CursorLocation);
				Pointer.Location = new(BackLabel.Location.X + xy.X, BackLabel.Location.Y + xy.Y);
			}
		}
	}
    public string WatermarkText
    {
        get
        {
            return BackWatermark.Text;
        }
        set
        {
            BackWatermark.Text = value;
            if (string.IsNullOrEmpty(Text))
            {
	            Pointer.Location = BackLabel.Location;
            }
        }
    }

    public event Func<Task>? OnNewLine;
    public event Func<Task>? OnRemoveLine;

    public override void SendKeyEvent(KeyboardKeyEventArgs KeyArgs)
    {
	    base.SendKeyEvent(KeyArgs);
	    if (KeyArgs.Key == Keys.Left && CursorLocation != 0)
	    {
		    CursorLocation--;
		    var f = BackLabel.GetCharLocation(CursorLocation);
		    Pointer.Location = BackLabel.Location + new Vector2i(f.X, f.Y);
		    Pointer.Visible = true;
	    }
	    if (KeyArgs.Key == Keys.Right && CursorLocation != Text.Length)
	    {
		    CursorLocation++;
		    var f = BackLabel.GetCharLocation(CursorLocation);
		    Pointer.Location = BackLabel.Location + new Vector2i(f.X, f.Y);
		    Pointer.Visible = true;
	    }
	    if (KeyArgs.Key is Keys.CapsLock or Keys.Menu or Keys.LeftSuper or Keys.RightSuper or Keys.End or Keys.Home or Keys.PageDown or Keys.PageUp or Keys.Insert or Keys.Up or Keys.Down or Keys.Left or Keys.Right) return;
	    if (KeyArgs.Key == Keys.Backspace)
	    {
		    if (!(Text.Length > 0)) return;
		    if (Text[CursorLocation- 1] == '\n')
		    {
			    Size = new(Size.X, Size.Y - (int)BackLabel.Font.PixelHeight);
			    if (OnRemoveLine is not null) OnRemoveLine.Invoke().Wait();
		    }
		    CursorLocation--;
		    Text = Text.Remove(CursorLocation, 1);
		    if (LetterRemoved is not null) LetterRemoved.Invoke().Wait();
		    var f = BackLabel.GetCharLocation(CursorLocation);
		    Pointer.Location = BackLabel.Location + new Vector2i(f.X, f.Y);
		    Pointer.Visible = true;
	    }
	    if (KeyArgs.Key == Keys.Delete)
	    {
		    if (CursorLocation == Text.Length) return;
		    if (Text[CursorLocation] == '\n')
		    {
			    Size = new(Size.X, Size.Y - (int)BackLabel.Font.PixelHeight);
			    if (OnRemoveLine is not null) OnRemoveLine.Invoke().Wait();
		    }
		    Text = Text.Remove(CursorLocation, 1);
		    if (LetterRemoved is not null) LetterRemoved.Invoke().Wait();
	    }

	    if (KeyArgs.Key == Keys.Enter)
	    {
		    if (AllowMultiLine && KeyArgs.Shift)
		    {
			    AddBlockDraw();
			    Size = new(Size.X, Size.Y + (int)BackLabel.Font.PixelHeight);
			    if (OnNewLine is not null) OnNewLine.Invoke().Wait();
			    RemoveBlockDraw();
			    Text += '\n';
			    CursorLocation++;
			    var f = BackLabel.GetCharLocation(CursorLocation);
			    Pointer.Location = BackLabel.Location + new Vector2i(f.X, f.Y);
			    Pointer.Visible = true;
		    }
		    else
		    {
			    
		    }
	    }



	    if (KeyArgs is { Key: Keys.V, Control: true } && Window is not null)
	    {
		    
		    try
		    {
			    SendClipEvent(Window.ClipboardString);
			    LetterPress?.Invoke().Wait();
		    }
		    catch
		    {
			    // ignored
		    }
	    }
	    if (KeyPress is not null) _ = KeyPress.Invoke(KeyArgs);
    }
}