using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using GraphicsManager.Objects.Core.Fonts;
using Luski.GUI.MainScreen.UI.Generic;
using Luski.GUI.MainScreen.UI.LuskiControls;
using Luski.GUI.MainScreen.UI.LuskiSettings;
using Luski.GUI.MainScreen.UI.PublicServers;
using OpenTK.Audio.OpenAL;
using OpenTK.Graphics.OpenGL4;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.Desktop;
using OpenTK.Windowing.GraphicsLibraryFramework;
using All = OpenTK.Platform.Windows.All;
using Window = GraphicsManager.Window;

namespace Luski.New.GUI;

public class RootWindow : Window
{
    private static readonly NativeWindowSettings Settings = new()
    {
        Title = "Luski",
        WindowBorder = WindowBorder.Resizable,
        APIVersion = new(3, 2),
        API = ContextAPI.OpenGL,
        StartFocused = true,
        ClientSize = new(1920, 1080),
        Icon = Globals.Icon,
        SharedContext = null
    };
    
    private ServerFlow ServerFlow;
    private ChannelSelector ChannelSelector;
    private PublicChat PublicChat;
    private UserControl ServerTitle;

    public RootWindow()
    : base(Settings)
    {
        base.UseSubFrames = false;
        base.UseLiveView = false;
        base.CustomF11 = false;

        #region Fonts
        
        Globals.RootWindow = this;
        try
        {
            Globals.DefaultFontFamly = FontFamily.LoadFontFamily(Globals.GetResource("Fonts.OpenSans.zip"), "OpenSans");
            Globals.DefaultFont = FontInteraction.Load(Globals.DefaultFontFamly);
            string fams = Path.Combine(Globals.LuskiPath, "FontFamilies");
            if (!Directory.Exists(fams)) Directory.CreateDirectory(fams);
            DirectoryInfo di = new DirectoryInfo(fams);
            
            foreach (FileInfo fam in di.GetFiles())
            {
                try
                {
                    if (fam.FullName.ToLower().EndsWith(".zip"))
                        Globals.DefaultFont.AddFamily(FontFamily.LoadFontFamily(File.Open(fam.FullName, FileMode.Open), fam.Name));
                }
                catch (Exception e)
                {
                    Console.WriteLine(e);
                }
            }
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }

        Globals.DefaultFont.PixelHeight = Globals.Settings.DefaultFontPX.ScaleFont();
        Globals.DefaultFont.FontSize = FontSize.Regular;
        Globals.TopTimeFont = Globals.DefaultFont.Clone();
        Globals.TopTimeFont.PixelHeight = Globals.Settings.TopTimeFontPX.ScaleFont();
        Globals.TopTimeFont.FontSize = FontSize.Regular;
        Globals.ServerRoleFont = Globals.DefaultFont.Clone();
        Globals.ServerRoleFont.PixelHeight = Globals.Settings.RoleSettingsFontPX.ScaleFont();
        Globals.ServerRoleFont.FontSize = FontSize.Regular;
        Globals.MessageFont = Globals.DefaultFont.Clone();
        Globals.MessageFont.PixelHeight = Globals.Settings.MessageFontPX.ScaleFont();
        Globals.MessageFont.ExtraLinePixels = Globals.Settings.MessageFontLineSpacePX.ScaleFont();
        Globals.MessageFont.FontSize = FontSize.Regular;
        Globals.SmallTimeFont = Globals.DefaultFont.Clone();
        Globals.SmallTimeFont.PixelHeight = ((uint)11).ScaleFont();
        Globals.SliderFont = Globals.DefaultFont.Clone();
        Globals.SliderFont.PixelHeight = ((uint)14).ScaleFont();
        Globals.LuskiTexture = TextureManager.GetTextureResource("Luski.png");
        
        #endregion
        
        ServerFlow = new()
        {
            BackgroundColor = new(26, 26, 26, 255),
            Size = new(68.ScaleInt(), CS.Y-52.ScaleInt()),
            Anchor = ObjectAnchor.Top | ObjectAnchor.Left | ObjectAnchor.Bottom
        };
        ServerTitle = new()
        {
            Location = new(ServerFlow.Size.X, 0),
            Size = new(307.ScaleInt(), 46.ScaleInt()),
            BackgroundColor = new(30, 30, 30, 255)
        };
        ChannelSelector = new()
        {
            BackgroundColor = new(34, 34, 34, 255),
            Location = new(ServerFlow.Size.X, ServerTitle.Size.Y + 2.ScaleInt()),
            Size = new(ServerTitle.Size.X, CS.Y - 54.ScaleInt() - ServerTitle.Size.Y),
            Anchor = ObjectAnchor.Top | ObjectAnchor.Left | ObjectAnchor.Bottom
        };
        PublicChat = new()
        {
            Location = new(ChannelSelector.Size.X + ServerFlow.Size.X, 0),
            Size = new(Size.X - ChannelSelector.Size.X - ServerFlow.Size.X, CS.Y),
            Anchor = ObjectAnchor.All
        };
        Rectangle setting = new(TextureManager.GetTextureResource("settings.png"))
        {
            Location = new(18.ScaleInt(), ServerFlow.Size.Y + 10.ScaleInt()),
            Size = new(32.ScaleInt()),
            Shader = Rectangle.DefaultAlphaShader[Context],
            BackgroundColor = Color4.Gray,
            Anchor = ObjectAnchor.Bottom | ObjectAnchor.Left,
            BlendOverride = true
        };
        setting.MouseEnter += _ =>
        {
            setting.BackgroundColor = Color4.White;
            return Task.CompletedTask;
        };
        setting.MouseLeave += _ =>
        {
            setting.BackgroundColor = Color4.Gray;
            return Task.CompletedTask;
        };
        setting.Clicked += SettingOnClicked;
        Controls.Add(setting);
        Controls.Add(ServerFlow);
        Controls.Add(ServerTitle);
        Controls.Add(ChannelSelector);
        Controls.Add(PublicChat);
    }
    
    private GlobalSettingsMenu? gsm;
    
    private Task SettingOnClicked(RenderObjectCore arg)
    {
        Console.WriteLine("SettingOnClicked");
        gsm ??= new();
        Controls.Add(gsm);
        Globals.RootWindow.DrawFrame();
        return Task.CompletedTask;
    }
    
    protected override void OnKeyDown(KeyboardKeyEventArgs e)
    {
        base.OnKeyDown(e);
        if (e.Key == Keys.F4)
        {
            if (!Context.IsCurrent) Context.MakeCurrent();
            Context.SwapBuffers();
            Console.WriteLine($"Force Swap");
        }
        else if (e.Key == Keys.F5)
        {
            ForceUpdate();
        }
        else if (e.Key == Keys.F8)
        {
            if (focused is null)
            {
                Console.WriteLine("There is not focused object");
            }
            else
            {
                Console.WriteLine($"Focused objet: {focused}\n\tLocation: {focused.Location}\n\tSize: {focused.Size}");
            }
        }
        else if (e.Key == Keys.F7)
        {
            if (HoveringControl is null)
            {
                Console.WriteLine("There is not hovering object");
            }
            else
            {
                Console.WriteLine($"Hovering objet: {HoveringControl}\n\tLocation: {HoveringControl.Location}\n\tSize: {HoveringControl.Size}");
            }
        }
        else if (e.Key == Keys.F9)
        {
            if (HoveringControl is IParent p)
            {
                Globals.PrintParent(p);
            }
            else
            {
                Globals.PrintParent(this);
            }
        }
        else if (e.Key == Keys.F10)
        {
            CheckParent(this);
        }
        
    }
}